const std = @import("std");

const c = @import("libvirt-c.zig").c;

pub fn handleError() VirError {
    const err = c.virGetLastError();
    // std.debug.print("err: {any}\n", .{err});
    return switch (err.*.code) {
        c.VIR_ERR_OK => VirError.OK,
        c.VIR_ERR_INTERNAL_ERROR => VirError.InternalError,
        c.VIR_ERR_NO_MEMORY => VirError.NoMemory,
        c.VIR_ERR_NO_SUPPORT => VirError.NoSupport,
        c.VIR_ERR_UNKNOWN_HOST => VirError.UnknownHost,
        c.VIR_ERR_NO_CONNECT => VirError.NoConnect,
        c.VIR_ERR_INVALID_CONN => VirError.InvalidConn,
        c.VIR_ERR_INVALID_DOMAIN => VirError.InvalidDomain,
        c.VIR_ERR_INVALID_ARG => VirError.InvalidArg,
        c.VIR_ERR_OPERATION_FAILED => VirError.OperationFailed,
        c.VIR_ERR_GET_FAILED => VirError.GetFailed,
        c.VIR_ERR_POST_FAILED => VirError.PostFailed,
        c.VIR_ERR_HTTP_ERROR => VirError.HttpError,
        c.VIR_ERR_SEXPR_SERIAL => VirError.SexprSerial,
        c.VIR_ERR_NO_XEN => VirError.NoXen,
        c.VIR_ERR_XEN_CALL => VirError.XenCall,
        c.VIR_ERR_OS_TYPE => VirError.OsType,
        c.VIR_ERR_NO_KERNEL => VirError.NoKernel,
        c.VIR_ERR_NO_ROOT => VirError.NoRoot,
        c.VIR_ERR_NO_TARGET => VirError.NoTarget,
        c.VIR_ERR_NO_NAME => VirError.NoName,
        c.VIR_ERR_NO_OS => VirError.NoOs,
        c.VIR_ERR_NO_DEVICE => VirError.NoDevice,
        c.VIR_ERR_DRIVER_FULL => VirError.DriverFull,
        c.VIR_ERR_CALL_FAILED => VirError.CallFailed,
        c.VIR_ERR_XML_ERROR => VirError.XmlError,
        c.VIR_ERR_DOM_EXIST => VirError.DomExist,
        c.VIR_ERR_OPERATION_DENIED => VirError.OperationDenied,
        c.VIR_ERR_OPEN_FAILED => VirError.OpenFailed,
        c.VIR_ERR_READ_FAILED => VirError.ReadFailed,
        c.VIR_ERR_PARSE_FAILED => VirError.ParseFailed,
        c.VIR_ERR_CONF_SYNTAX => VirError.ConfSyntax,
        c.VIR_ERR_WRITE_FAILED => VirError.WriteFailed,
        c.VIR_ERR_XML_DETAIL => VirError.XmlDetail,
        c.VIR_ERR_INVALID_NETWORK => VirError.InvalidNetwork,
        c.VIR_ERR_NETWORK_EXIST => VirError.NetworkExist,
        c.VIR_ERR_SYSTEM_ERROR => VirError.SystemError,
        c.VIR_ERR_RPC => VirError.Rpc,
        c.VIR_ERR_GNUTLS_ERROR => VirError.GnutlsError,
        c.VIR_WAR_NO_NETWORK => VirError.WarNoNetwork, // ???
        c.VIR_ERR_NO_DOMAIN => VirError.NoDomain,
        c.VIR_ERR_NO_NETWORK => VirError.NoNetwork,
        c.VIR_ERR_INVALID_MAC => VirError.InvalidMac,
        c.VIR_ERR_AUTH_FAILED => VirError.AuthFailed,
        c.VIR_ERR_INVALID_STORAGE_POOL => VirError.InvalidStoragePool,
        c.VIR_ERR_INVALID_STORAGE_VOL => VirError.InvalidStorageVol,
        c.VIR_WAR_NO_STORAGE => VirError.WarNoStorage, // ???
        c.VIR_ERR_NO_STORAGE_POOL => VirError.NoStoragePool,
        c.VIR_ERR_NO_STORAGE_VOL => VirError.NoStorageVol,
        c.VIR_WAR_NO_NODE => VirError.WarNoNode, // ???
        c.VIR_ERR_INVALID_NODE_DEVICE => VirError.InvalidNodeDevice,
        c.VIR_ERR_NO_NODE_DEVICE => VirError.NoNodeDevice,
        c.VIR_ERR_NO_SECURITY_MODEL => VirError.NoSecurityModel,
        c.VIR_ERR_OPERATION_INVALID => VirError.OperationInvalid,
        c.VIR_WAR_NO_INTERFACE => VirError.WarNoInterface, // ???
        c.VIR_ERR_NO_INTERFACE => VirError.NoInterface,
        c.VIR_ERR_INVALID_INTERFACE => VirError.InvalidInterface,
        c.VIR_ERR_MULTIPLE_INTERFACES => VirError.MultipleInterfaces,
        c.VIR_WAR_NO_NWFILTER => VirError.WarNoNwfilter, // ???
        c.VIR_ERR_INVALID_NWFILTER => VirError.InvalidNwfilter,
        c.VIR_ERR_BUILD_FIREWALL => VirError.BuildFirewall,
        c.VIR_WAR_NO_SECRET => VirError.WarNoSecret, // ???
        c.VIR_ERR_INVALID_SECRET => VirError.InvalidSecret,
        c.VIR_ERR_CONFIG_UNSUPPORTED => VirError.ConfigUnsupported,
        c.VIR_ERR_OPERATION_TIMEOUT => VirError.OperationTimeout,
        c.VIR_ERR_MIGRATE_PERSIST_FAILED => VirError.MigratePersistFailed,
        c.VIR_ERR_HOOK_SCRIPT_FAILED => VirError.HookScriptFailed,
        c.VIR_ERR_INVALID_DOMAIN_SNAPSHOT => VirError.InvalidDomainSnapshot,
        c.VIR_ERR_NO_DOMAIN_SNAPSHOT => VirError.NoDomainSnapshot,
        c.VIR_ERR_INVALID_STREAM => VirError.InvalidStream,
        c.VIR_ERR_ARGUMENT_UNSUPPORTED => VirError.ArgumentUnsupported,
        c.VIR_ERR_STORAGE_PROBE_FAILED => VirError.StorageProbeFailed,
        c.VIR_ERR_STORAGE_POOL_BUILT => VirError.StoragePoolBuilt,
        c.VIR_ERR_SNAPSHOT_REVERT_RISKY => VirError.SnapshotReverRisky,
        c.VIR_ERR_OPERATION_ABORTED => VirError.OperationAborted,
        c.VIR_ERR_AUTH_CANCELLED => VirError.AuthCancelled,
        c.VIR_ERR_NO_DOMAIN_METADATA => VirError.NoDomainMetadata,
        c.VIR_ERR_MIGRATE_UNSAFE => VirError.MigrateUnsafe,
        c.VIR_ERR_OVERFLOW => VirError.Overflow,
        c.VIR_ERR_BLOCK_COPY_ACTIVE => VirError.BlockCopyActive,
        c.VIR_ERR_OPERATION_UNSUPPORTED => VirError.OperationUnsupported,
        c.VIR_ERR_SSH => VirError.Ssh,
        c.VIR_ERR_AGENT_UNRESPONSIVE => VirError.AgentUnresponsive,
        c.VIR_ERR_RESOURCE_BUSY => VirError.ResourceBusy,
        c.VIR_ERR_ACCESS_DENIED => VirError.AccessDenied,
        c.VIR_ERR_DBUS_SERVICE => VirError.DbusService,
        c.VIR_ERR_STORAGE_VOL_EXIST => VirError.StorageVolExist,
        c.VIR_ERR_CPU_INCOMPATIBLE => VirError.CpuIncompatible,
        c.VIR_ERR_XML_INVALID_SCHEMA => VirError.XmlInvalidSchema,
        c.VIR_ERR_MIGRATE_FINISH_OK => VirError.MigrateFinishOk,
        c.VIR_ERR_AUTH_UNAVAILABLE => VirError.AuthUnavailable,
        c.VIR_ERR_NO_SERVER => VirError.NoServer,
        c.VIR_ERR_NO_CLIENT => VirError.NoClient,
        c.VIR_ERR_AGENT_UNSYNCED => VirError.AgentUnsynced,
        c.VIR_ERR_LIBSSH => VirError.Libssh,
        c.VIR_ERR_DEVICE_MISSING => VirError.DeviceMissing,
        c.VIR_ERR_INVALID_NWFILTER_BINDING => VirError.InvalidNwfilterBinding,
        c.VIR_ERR_NO_NWFILTER_BINDING => VirError.NoNwfilterBinding,
        c.VIR_ERR_INVALID_DOMAIN_CHECKPOINT => VirError.InvalidDomainCheckpoint,
        c.VIR_ERR_NO_DOMAIN_CHECKPOINT => VirError.NoDomainCheckpoint,
        c.VIR_ERR_NO_DOMAIN_BACKUP => VirError.NoDomainBackup,
        c.VIR_ERR_INVALID_NETWORK_PORT => VirError.InvalidNetworkPort,
        c.VIR_ERR_NETWORK_PORT_EXIST => VirError.NetworkPortExist,
        c.VIR_ERR_NO_NETWORK_PORT => VirError.NoNetworkPort,
        c.VIR_ERR_NO_HOSTNAME => VirError.NoHostname,
        c.VIR_ERR_CHECKPOINT_INCONSISTENT => VirError.CheckpointInconsistent,
        c.VIR_ERR_MULTIPLE_DOMAINS => VirError.MultipleDomains,
        c.VIR_ERR_NO_NETWORK_METADATA => VirError.NoNetworkMetadata,
        else => VirError.OK,
    };
}

pub const VirError = error{
    OK,
    InternalError,
    NoMemory,
    NoSupport,
    UnknownHost,
    NoConnect,
    InvalidConn,
    InvalidDomain,
    InvalidArg,
    OperationFailed,
    GetFailed,
    PostFailed,
    HttpError,
    SexprSerial,
    NoXen,
    XenCall,
    OsType,
    NoKernel,
    NoRoot,
    NoSource,
    NoTarget,
    NoName,
    NoOs,
    NoDevice,
    NoXenstore,
    DriverFull,
    CallFailed,
    XmlError,
    DomExist,
    OperationDenied,
    OpenFailed,
    ReadFailed,
    ParseFailed,
    ConfSyntax,
    WriteFailed,
    XmlDetail,
    InvalidNetwork,
    NetworkExist,
    SystemError,
    Rpc,
    GnutlsError,
    WarNoNetwork, // ???
    NoDomain,
    NoNetwork,
    InvalidMac,
    AuthFailed,
    InvalidStoragePool,
    InvalidStorageVol,
    WarNoStorage, // ???
    NoStoragePool,
    NoStorageVol,
    WarNoNode, // ???
    InvalidNodeDevice,
    NoNodeDevice,
    NoSecurityModel,
    OperationInvalid,
    WarNoInterface, // ???
    NoInterface,
    InvalidInterface,
    MultipleInterfaces,
    WarNoNwfilter, // ???
    InvalidNwfilter,
    BuildFirewall,
    WarNoSecret, // ???
    InvalidSecret,
    ConfigUnsupported,
    OperationTimeout,
    MigratePersistFailed,
    HookScriptFailed,
    InvalidDomainSnapshot,
    NoDomainSnapshot,
    InvalidStream,
    ArgumentUnsupported,
    StorageProbeFailed,
    StoragePoolBuilt,
    SnapshotReverRisky,
    OperationAborted,
    AuthCancelled,
    NoDomainMetadata,
    MigrateUnsafe,
    Overflow,
    BlockCopyActive,
    OperationUnsupported,
    Ssh,
    AgentUnresponsive,
    ResourceBusy,
    AccessDenied,
    DbusService,
    StorageVolExist,
    CpuIncompatible,
    XmlInvalidSchema,
    MigrateFinishOk,
    AuthUnavailable,
    NoServer,
    NoClient,
    AgentUnsynced,
    Libssh,
    DeviceMissing,
    InvalidNwfilterBinding,
    NoNwfilterBinding,
    InvalidDomainCheckpoint,
    NoDomainCheckpoint,
    NoDomainBackup,
    InvalidNetworkPort,
    NetworkPortExist,
    NoNetworkPort,
    NoHostname,
    CheckpointInconsistent,
    MultipleDomains,
    NoNetworkMetadata,
};
